% Test particle + Liouville mapping to generated electron distriubtions in Figure 4 of the manuscript
% RH4 method is used to trace the elctron trajectory
% It may take a long time to run (> 10 min)
% Written by chgao/bbtang
clear;

 %%  1. basic
energypad = [6.52,8.54,11.17,14.63,19.15,25.07,32.81,42.95,56.23,73.60,96.34,126.12,165.09,216.11,282.89,370.31,484.74,634.54,830.63,1087.31,1423.32,1863.16,2438.92,3192.61,4179.20,5470.68,7161.25,9374.25,12271.12,16063.20,21027.10,27525];
 % Energytable used in MMS
 %[eV]

 
 
 Mp = 9.1094e-31;             % electron mass
 Qe = -1.6022e-19;            % particle charge
  V = (2*energypad*(-Qe)/Mp).^0.5/1.e6;
 %Convert the energy into speed
 %[10^3 km/s]

 
    z=[];%Variable name of result

    
    We = 50    ;                      % [eV]
    T  = 60    ;                      % [eV]
    r  = 871   ;                      % [m]
    n1 = 0*1.e6;                      % [m^-3]
    n2 = 6*1.e6;                      % [m^-3]
    d1 = 0.1*r;                       % n=n1*(tanh(RR(i,1)/d1)+1)*0.5+n2
    B0amp = 30 ;                      % [nT]
    B1 = 0;                           % [nT] 
    B2 = 30 ;                         % [nT] 
    d2 = 0.1*r   ;
    d3 = 0.1*r  ;                     % B=B1*(-tanh((RR(i,1)+d2)/d3)+1)*0.5+B2
    E0amp = 100 ;                     % [mV/m] Peak value of electric field
    X  = 615;                         % [m]
    Q  = X/2.355;                     % E=E0amp*exp(-(RR(i,1)^2)/(2*Q^2));
    y0 = 0;                           % [m]
    z0 = 0;                           % [m]
    vz0= 0;                           % [m]
    clim=[-29 -25];                   % colorbar
    v  = 34000;                   % [m/s] 
    case_number = 1;                  % case_number 
%%  2. make data
for ii=1:4                       % 4 location
for a=1:32 %For 32 azimuthal direction 
for b=1:19 % actually the first 19 energy channels are used ( do not using all 32 energy channeles  to improve efficiency)

theat=((a-1)*pi/16)+pi/32;
Vx=1000000*V(b)*cos(theat);
Vy=1000000*V(b)*sin(theat);
x0 = -1165+v*0.0075*(ii-1);     % 4 different location
    
    qtemp = [x0; Vx; y0; Vy; z0; vz0];
 %   fce = 28 * B0amp;
    fce = abs(Qe) * norm(B0amp)*1.e-9/(Mp*2*pi);
    rhoe = 3.4 * sqrt(We) / B0amp;  
     
    trange = [0, -0.03/fce]; 
    t0 = trange(1);
    RR = [qtemp(1),qtemp(3),qtemp(5)];
    VV = [qtemp(2),qtemp(4),qtemp(6)];
    tt = t0;
    
    tic;
    while (t0 >trange(2))
        % time step
        dt = -1/fce/50; 
         
        % rk4
        k1 = RHT(t0, qtemp, E0amp, B1 ,B2, X, d2, d3);
        k2 = RHT(t0 + .5*dt, qtemp + k1 * 0.5*dt, E0amp, B1 ,B2, X, d2, d3);
        k3 = RHT(t0 + .5*dt, qtemp + k2 * 0.5*dt, E0amp, B1 ,B2, X, d2, d3);
        k4 = RHT(t0 + dt, qtemp + k3 * dt, E0amp, B1 ,B2, X, d2, d3);
        q1 = qtemp + 1/6*(k1 + 2*k2 + 2*k3 + k4) * dt;
        
        % update
        tt = [tt; tt(end)+dt];
        rr1 = [q1(1), q1(3), q1(5)];
        RR = [RR; rr1];
        vv1 = [q1(2), q1(4), q1(6)];
        VV = [VV; vv1];
        
        % prepare for next step
        t0 = tt(end);
        qtemp = q1;
    end
  
   
   for i=1:length(VV)
   E=E0amp*exp(-(RR(i,1)^2)/(2*Q^2));
   B=B1*(-tanh((RR(i,1)+d2)/d3)+1)*0.5+B2;
   n=n1*(tanh(RR(i,1)/d1)+1)*0.5+n2;
   VTOT=VV(:, 1).^2 + (VV(:, 2)+1*E*1.e6/B).^2+ VV(:, 3).^2;
   f(i)=n*(Mp/(2*pi*T*(-Qe)))^(3/2)*exp(-(Mp*(VTOT(i)))/(2*T*(-Qe)));           % Maxwell distribution
   end
   z(b,a,ii)=log10(mean(f))-12;
   Vtot = sqrt(VV(:, 1).^2 + VV(:, 2).^2 + VV(:, 3).^2);     
    Wetot = 1/2 * Vtot.^2 * Mp / (-Qe);
toc;
end

end
end
%% plot
z0 = test_reshape(z);
subplot(1,2,2)
polar3d_MMS(z0);
axis equal
view(0,90);
xSize=1300; ySize=550;
set(gcf,'Position',[10 10 xSize ySize]);

xwidth = 0.3;         
ywidth = 0.6;

c_eval('set(gca, ''position'', [0.59 0.25 xwidth ywidth]);', 1);
colormap('jet');shading interp;
set(gca,'XLim',[-12 12],'fontsize',24);
set(gca,'XTick',[-10:10:10],'fontsize',24);
set(gca,'XTickLabel',[-10:10:10],'fontsize',24);
set(gca,'YLim',[-12 12],'fontsize',24);
set(gca,'YTick',[-10:10:10],'fontsize',24);
set(gca,'YTickLabel',[-10:10:10],'fontsize',24);
xlabel('v_{E}(10^3km/s)','fontsize',24)
ylabel('v_{ExB}(10^3km/s)','fontsize',24)
h=colorbar();
caxis(clim);
ylabel(colorbar,'log_{10}f_e(s^3/cm^6)','fontsize',24);
%irf_legend(gca,'(c2)',[-0.15 1.03],'fontsize',26,'color','k')
subplot(1,2,1)
backgroundf(case_number) ;  